# frozen_string_literal: true

# Compute the total size taken in Container Registry by a top-level group
#
# The results are stored on ProjectStatistics instances for all projects in
# the group *and* its subgroup, taking layer reuse into account across all
# the projects.
#
# Deduplication of layers potentially shared with other groups does not
# take place.
#
# We decide arbitrarily here that each project contributes equally to
# each layer. This is arbitrary, because a project that references, e.g,
# debian:buster for the 3rd time does not create new storage needs.
# Eventually, when layers get less and less referenced, the cost for projects
# using them will climb up. To continue on the example, assume that
# the two other projects move on to debian:bullseye. The 3rd project will now
# carry the whole weight of debian:buster.
# Still, once these are sum up for the top-level group by summing
# group statistics recursively (these are by definition themselves summing
# direct projects), one get the total size  of all layers that are referenced
# somewhere in that group.
# That is also what Namespaces::StatisticsRefresherService,
# iself launched by Namespaces::RootStatisticsWorker will come up to.
#
# If one does not care about deduplicating with other top-level groups,
# this can serve as a base for billing.
# rubocop:disable CodeReuse/ActiveRecord
module Namespaces
  class ContainerRegistryStatsService < Groups::BaseService
    attr_accessor :namespace

    def initialize(namespace)
      @namespace = namespace
    end

    def execute
      raise StandardError, "Meaningful for top-level namespaces only" unless namespace.parent_id.nil?

      start = Time.new.utc
      Gitlab::AppLogger.info(
        message: "Namespaces::ContainerProjectStatsService starting for top-level namespace",
        namespace_id: namespace.id,
        namespace_path: namespace.path
      )

      project_size_shares.each_pair do |pid, size|
        project_stats = ProjectStatistics.find_or_create_by(project_id: pid)
        project_stats.container_registry_size_share = size.round
        project_stats.save!
      end

      Gitlab::AppLogger.info(
        message: "Namespaces::ContainerProjectStatsService done for top-level namespace",
        namespace_id: namespace.id,
        namespace_path: namespace.path,
        duration_seconds: Time.new.utc - start
      )
    end

    def container_repositories
      ns_ids =
        # namespace.is_a Group should also work, but let's be cautious
        if namespace.type == "Group"
          namespace.self_and_descendants.select(:id)
        else
          namespace.id
        end

      ContainerRepository
        .joins(project: :namespace)
        .where(namespaces: { id: ns_ids })
    end

    # Return a Hash mapping each layer to its size and an Array of projects
    # that reference it.
    #
    # Layers are actually represented by their digest (usually a SHA256)
    def all_layers
      layers = {}
      container_repositories.each do |cont_repo|
        cont_repo.tags.each do |tag|
          Gitlab::AppLogger.debug(
            message: "Namespaces::ContainerProjectStatsService: examining Docker image",
            project_id: cont_repo.project_id,
            slug: "#{cont_repo.name}",
            tag: "#{tag.name}"
          )
          tag.layers.each do |layer|
            layer_info = layers[layer.digest]
            if layer_info.nil?
              layers[layer.digest] = [layer.size, Set[cont_repo.project_id]]
            else
              layer_info[1] << cont_repo.project_id
            end
          end
        end
      end
      layers
    end

    # Return a Hash mapping project ids to the size they take in the Registry.
    def project_size_shares
      sizes = {}
      all_layers.each_value do |size, project_ids|
        # we don't care very much about exactness of division
        # in the grand scheme of things (how much would a byte be
        # billed anyway?)
        # but it'll be useful for unit tests and other checks that the sums
        # matches (up to the usual floating point rounding issues)
        layer_size_per_project = Float(size) / project_ids.length
        project_ids.each do |proj_id|
          project_size = sizes[proj_id]
          sizes[proj_id] =
            if project_size.nil?
              layer_size_per_project
            else
              project_size + layer_size_per_project
            end
        end
      end
      sizes
    end
  end
end
