# frozen_string_literal: true

class ForceNotesRerender < ActiveRecord::Migration[6.0]
  include Gitlab::Database::MigrationHelpers

  DOWNTIME = false

  disable_ddl_transaction!

  def up
    # In the update_column_in_batches block below, `table` and `query` are Arel objects.
    # Arel representation for UPDATE queries (UpdateManager) does not
    # have a `#join`, neither a `#from` method, so we're resorting to a subquery to
    # restrict the migration to notes on projects with the `hg_git` VCS type
    # It is possible that later versions would overcome these limitations.
    # This way less efficient than a JOIN, but it seems to be the only way to do
    # this currently, and is mitigated by processing in batches.

    # It doesn't seem to be straightforward to use an Arel `SelectManager` in the WHERE clause,
    # so we're writing the SQL directly (which is simple enough not to create additional
    # maintenance problems). The parentheses are crucial.
    sql_vcs_type_subquery = Arel.sql(
      "(SELECT vcs_type FROM projects WHERE projects.id = notes.project_id)")

    update_column_in_batches(:notes, :note_html, nil) do |table, query|
      query.where(sql_vcs_type_subquery.eq("hg_git"))
    end
  end

  def down
    # no-op: on one hand, there is no way to revert the migration,
    # on the other hand, the affected column is nothing more than a cache.
  end
end
