# frozen_string_literal: true

module Gitlab
  module Checks
    class SingleChangeAccess
      ATTRIBUTES = %i[user_access project skip_authorization
                      protocol oldrev newrev ref
                      branch_name tag_name logger commits].freeze

      attr_reader(*ATTRIBUTES)

      def initialize(
        change, user_access:, project:,
        accept_mr_iid:,
        protocol:, logger:
      )
        @oldrev, @newrev, @ref = change.values_at(:oldrev, :newrev, :ref)
        @branch_name = Gitlab::Git.branch_name(@ref)
        @tag_name = Gitlab::Git.tag_name(@ref)
        @user_access = user_access
        @project = project
        @protocol = protocol
        @accept_mr_iid = accept_mr_iid

        @logger = logger
        @logger.append_message("Running checks for ref: #{@branch_name || @tag_name}")
      end

      # If the pre-receive is ultimately due to a merge from Rails,
      # return the corresponding MR target branch name.
      #
      # The proof it comes from Rails is the presence of @accept_mr_iid.
      # In other cases, `nil` is returned. On Mercurial that will lead
      # to the harsher (direct) push checks running.
      def target_branch_for_mr
        return if @accept_mr_iid.nil?

        # rubocop: disable CodeReuse/ActiveRecord
        # gracinet, could not find a direct helper for this in 10 seconds
        # this is a very simple call (no perf issue) anyway
        mr = project.merge_requests.find_by(iid: @accept_mr_iid)
        # rubocop: enable CodeReuse/ActiveRecord

        if mr.nil?
          Gitlab::AppLogger.warn(
            method: "Gitlab::Check::ChangeAccess#target_branch_for_mr",
            message: "No such Merge Request",
            merge_request_iid: @accept_mr_iid,
            project_id: project.id
          )
        end

        mr&.target_branch
      end

      def validate!
        ref_level_checks
        # Check of commits should happen as the last step
        # given they're expensive in terms of performance
        commits_check

        true
      end

      def commits
        @commits ||= project.repository.new_commits(newrev)
      end

      protected

      def ref_level_checks
        Gitlab::Checks::PushCheck.new(self).validate!
        Gitlab::Checks::BranchCheck.new(self).validate!
        Gitlab::Checks::TagCheck.new(self).validate!
      end

      def commits_check
        Gitlab::Checks::DiffCheck.new(self).validate!
      end
    end
  end
end

Gitlab::Checks::SingleChangeAccess.prepend_mod_with('Gitlab::Checks::SingleChangeAccess')
