# frozen_string_literal: true

module Heptapod::NativeMercurial
  # Native Mercurial Migrator
  #
  # This is responsible for migrating a mercurial project from hg_git backend to native hg.
  class Migrator
    SHA_MAX_LEN = 40
    SHA_REGEX = '\b[0-9a-f]{7,40}\b'

    def initialize(project)
      @hg_sha_map = project.repository.hg_sha_map
      @project = project
    end

    def migrate!
      Gitlab::AppLogger.info(
        message: "Starting native hg migration",
        project_id: @project.id,
        project_full_path: @project.full_path
      )

      migrate_issues
      migrate_merge_requests
      migrate_notes
      migrate_releases

      # ensure all gitlab specific state files are present
      # TODO: replace following command with `hpd-ensure-all-gitlab-specific-state-files` once it's available
      output, status = @project.repository.hg_call(
        ["hpd-ensure-all-gitlab-refs"], force_system_user: true
      )
      raise StandardError, output.strip unless status == 0

      @project.vcs_type = 'hg'
      @project.save!

    rescue StandardError => err
      Gitlab::AppLogger.error(
        message: "Native mercurial migration failed: #{err}",
        backtrace: Gitlab::BacktraceCleaner.clean_backtrace(err.backtrace),
        project_id: @project.id,
        project_full_path: @project.full_path
      )
    end

    def migrate_issues
      @project.issues.each do |issue|
        issue.title = replace_git_shas_with_hg(issue.title)
        issue.title_html = nil
        issue.description = replace_git_shas_with_hg(issue.description)
        issue.description_html = nil
        issue.save!
      end
    end

    def migrate_notes
      notes_service = Notes::BaseService.new(@project)
      @project.notes.each do |note|
        note.note = replace_git_shas_with_hg(note.note)
        note.note_html = nil
        # clear noteable diffs cache
        notes_service.clear_noteable_diffs_cache(note)
        note.save!
      end
    end

    def migrate_releases
      @project.releases.each do |rl|
        rl.sha = replace_git_shas_with_hg(rl.sha)
        rl.description = replace_git_shas_with_hg(rl.description)
        rl.description_html = nil
        rl.save!
      end
    end

    def migrate_merge_requests
      @project.merge_requests.each do |mr|
        mr.title = replace_git_shas_with_hg(mr.title)
        mr.title_html = nil
        mr.description = replace_git_shas_with_hg(mr.description)
        mr.description_html = nil

        # update all the columns ending with '_sha'
        mr.merge_commit_sha = replace_git_shas_with_hg(mr.merge_commit_sha)
        mr.rebase_commit_sha = replace_git_shas_with_hg(mr.rebase_commit_sha)
        mr.in_progress_merge_commit_sha = (
          replace_git_shas_with_hg(mr.in_progress_merge_commit_sha)
        )
        mr.squash_commit_sha = replace_git_shas_with_hg(mr.squash_commit_sha)
        mr.merge_ref_sha = replace_git_shas_with_hg(mr.merge_ref_sha)
        migrate_merge_request_diffs(mr)
        mr.save!
      end
    end

    def migrate_merge_request_diffs(mr)
      mr.merge_request_diffs.each do |diff|
        diff.base_commit_sha = replace_git_shas_with_hg(diff.base_commit_sha)
        diff.head_commit_sha = replace_git_shas_with_hg(diff.head_commit_sha)
        diff.start_commit_sha = replace_git_shas_with_hg(diff.start_commit_sha)
        diff.save!
      end
      # clear merge request diffs cache
      mr.diffs.clear_cache
    end

    private

    def replace_git_shas_with_hg(text)
      return unless text.present? && @hg_sha_map.present?

      git_shas = @hg_sha_map.keys
      text.gsub(/#{SHA_REGEX}/) do |match|
        if match.length == SHA_MAX_LEN && git_shas.include?(match)
          @hg_sha_map[match]
        else
          full_git_sha = lambda {
            git_shas.each do |sha|
              if sha.starts_with? match
                return sha
              end
            end
            nil
          }.call
          full_git_sha.nil? ? match : @hg_sha_map[full_git_sha]
        end
      end
    end
  end
end
